#!/usr/bin/env python
import pytest
from dios import *
from dios.lib import _OP1_MAP, _OP2_DIV_MAP, _OP2_ARITH_MAP, _OP2_BOOL_MAP, _OP2_COMP_MAP

import pandas as pd
import numpy as np
from copy import deepcopy
from pandas.tests.series.conftest import datetime_series

__author__ = "Bert Palm"
__email__ = "bert.palm@ufz.de"
__copyright__ = "Copyright 2018, Helmholtz-Centrum für Umweltforschung GmbH - UFC"


def get_dios(ser, i):
    dios = DictOfSeries()
    for i in range(i):
        dios[f'c{i}'] = ser.copy() * (i + 1) // 2
    return dios


EE = [[], [], []]

O = [[0, 0, 0], [0, 0, 0], [0]]
I = [[1, 1, 1], [1, 1, 1], [1]]
# A = [[1, 2, 3], [4, 5, 6]]
# B = [[0, 2, 2], [5, 5, 5]]
# C = [[3, 2, 0], [1, 0, 3]]
# D = [[6, 5, 4], [3, 2, 1]]
A = [[1], [2, 3], [4, 5, 6]]
B = [[0, 2, 2], [5], [5, 5]]
C = [[3, 2, 0], [1, 0, 3], [0,0,0]]
D = [[6], [2], [9]]


DATA23 = [O, I, A, B, C, D]
ALL = [EE, O, I, A, B, C, D]


OP2 = list(_OP2_ARITH_MAP) + list(_OP2_COMP_MAP) + list(_OP2_BOOL_MAP) + list(_OP2_DIV_MAP)
OP1 = list(_OP1_MAP)


def _diosmatr(mlist):
    l = []
    for m in mlist:
        m = np.array(m)
        l.append(DictOfSeries(m.copy()))
    return tuple(l)


@pytest.fixture()
def dictofseries(datetime_series):
    return get_dios(datetime_series, 5)


def test_len__len__(datetime_series, maxlen=10):
    dios = DictOfSeries()
    assert len(dios) == 0

    for i in range(maxlen):
        dios[f'c{i}'] = datetime_series.copy()
        assert len(dios) == i + 1

    for i in reversed(range(maxlen)):
        assert len(dios) == i + 1
        del dios[f'c{i}']

    assert len(dios) == 0


def test_copy_copy_empty__copy__(dictofseries):
    dios = dictofseries.copy()
    shallow = dios.copy(deep=False)
    deep = dios.copy(deep=True)
    empty = dios.copy_empty()

    assert dios is not shallow
    assert dios is not deep
    assert dios is not empty

    assert dios.itype == shallow.itype
    assert dios.itype == deep.itype
    assert dios.itype == empty.itype

    for i in dios:
        assert dios[i].index is shallow[i].index
        assert dios[i].index is not deep[i].index
        dios[i][0] = 999999
        assert dios[i][0] == shallow[i][0]
        assert dios[i][0] != deep[i][0]


@pytest.mark.parametrize('left', _diosmatr(DATA23))
@pytest.mark.parametrize('right', _diosmatr(DATA23))
def test__eq__(left, right):
    a, b = left, right
    _test = a == b
    res = [entry for col in _test for entry in _test[col]]
    e1 = [entry for col in a for entry in a[col]]
    e2 = [entry for col in b for entry in b[col]]
    for i in range(len(res)):
        exp = e1[i] == e2[i]
        assert res[i] == exp


@pytest.mark.parametrize('left', _diosmatr(DATA23))
@pytest.mark.parametrize('right', _diosmatr(DATA23))
@pytest.mark.parametrize('op', _OP2_COMP_MAP)
def test_all(left, right, op):
    a, b = left, right
    ser = op(a,a).all()
    res = [e for e in ser]
    exp = [op(a[col],a[col]).all() for col in a]
    for i in range(len(res)):
        assert res[i] == exp[i]


@pytest.mark.parametrize('left', _diosmatr(ALL))
@pytest.mark.parametrize('right', _diosmatr(ALL))
@pytest.mark.parametrize('op', OP2)
def test__op2__(left, right, op):
    try:
        a, b = left, right
        test = op(a, b)
        # res = [entry for col in test for entry in test[col]]
        for c in test:
            for j in test[c].index:
                exp = op(a[c][j],b[c][j])
                res = test[c][j]
                if not np.isfinite(res):
                    pytest.mark.skip('test not support non-finite values')
                    return
                assert res == exp

    except ZeroDivisionError:
        pytest.mark.skip('ZeroDivisionError')


@pytest.mark.parametrize('data', _diosmatr(ALL))
@pytest.mark.parametrize('op', OP1)
def test__op1__(data, op):
    test = op(data)
    res = [entry for col in test for entry in test[col]]
    e = [entry for col in data for entry in data[col]]
    for i in range(len(res)):
        exp = op(e[i])
        assert res[i] == exp
